@@ -1,7 +1,7 @@
 //
 // Printer IPP processing for the Printer Application Framework
 //
-// Copyright © 2019-2024 by Michael R Sweet.
+// Copyright © 2019-2025 by Michael R Sweet.
 // Copyright © 2010-2019 by Apple Inc.
 //
 // Licensed under Apache License v2.0.  See the file "LICENSE" for more
@@ -541,16 +541,16 @@ _papplPrinterCopyAttributesNoLock(
 
     num_values = 0;
 
-    if (httpAddrIsLocalhost(httpGetAddress(client->http)) || !papplSystemGetTLSOnly(client->system))
+    if (!httpAddrIsLocalhost(httpGetAddress(client->http)) && !(client->system->options & PAPPL_SOPTIONS_NO_TLS))
     {
-      httpAssembleURI(HTTP_URI_CODING_ALL, uris[num_values], sizeof(uris[0]), "ipp", NULL, client->host_field, client->host_port, printer->resource);
+      httpAssembleURI(HTTP_URI_CODING_ALL, uris[num_values], sizeof(uris[0]), "ipps", NULL, client->host_field, client->host_port, printer->resource);
       values[num_values] = uris[num_values];
       num_values ++;
     }
 
-    if (!httpAddrIsLocalhost(httpGetAddress(client->http)) && !(client->system->options & PAPPL_SOPTIONS_NO_TLS))
+    if (httpAddrIsLocalhost(httpGetAddress(client->http)) || !papplSystemGetTLSOnly(client->system))
     {
-      httpAssembleURI(HTTP_URI_CODING_ALL, uris[num_values], sizeof(uris[0]), "ipps", NULL, client->host_field, client->host_port, printer->resource);
+      httpAssembleURI(HTTP_URI_CODING_ALL, uris[num_values], sizeof(uris[0]), "ipp", NULL, client->host_field, client->host_port, printer->resource);
       values[num_values] = uris[num_values];
       num_values ++;
     }
@@ -609,8 +609,8 @@ _papplPrinterCopyAttributesNoLock(
     {
       static const char * const uri_authentication_basic[] =
       {					// uri-authentication-supported values
-	"none",
-	"basic"
+	"basic",
+	"none"
       };
 
       ippAddStrings(client->response, IPP_TAG_PRINTER, IPP_CONST_TAG(IPP_TAG_KEYWORD), "uri-authentication-supported", 2, NULL, uri_authentication_basic);
@@ -790,7 +790,9 @@ _papplPrinterIsAuthorized(
   if (code == HTTP_STATUS_CONTINUE && client->job && client->job->username && strcmp(client->username, client->job->username))
   {
     // Not the owner, try authorizing with admin group...
+    _papplRWLockRead(client->system);
     code = _papplClientIsAuthorizedForGroup(client, true, client->system->admin_group, client->system->admin_gid);
+    _papplRWUnlock(client->system);
   }
 
   if (code == HTTP_STATUS_CONTINUE)
@@ -1604,7 +1606,9 @@ ipp_get_jobs(pappl_client_t *client)	// I - Client
       ippAddSeparator(client->response);
 
     count ++;
+    _papplRWLockRead(job);
     _papplJobCopyAttributesNoLock(job, client, ra);
+    _papplRWUnlock(job);
   }
 
   cupsArrayDelete(ra);
